package de.juplo.facebook.errors;


import de.juplo.facebook.errors.GraphApiException.Type;
import okhttp3.mockwebserver.MockResponse;
import okhttp3.mockwebserver.MockWebServer;
import org.junit.After;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;



/**
 *
 * @author Kai Moritz
 */
public class GraphApiErrorResponseErrorHandlerIntegrationTest
{
  private static final Logger LOG =
      LoggerFactory.getLogger(GraphApiErrorResponseErrorHandlerIntegrationTest.class);

  private MockWebServer server;
  private String uri;
  private RestTemplate clientTemplate;


	@Before
	public void setup()
  {
    server = new MockWebServer();
    uri = server.url("/egal").toString();
    clientTemplate = new RestTemplate();
    clientTemplate.setErrorHandler(new GraphApiErrorResponseErrorHandler(clientTemplate.getErrorHandler()));
	}

	@After
	public void shutdown() throws Exception
  {
    this.server.shutdown();
	}


  @Test
  public void testNoError()
  {
    //server
    //    .enqueue(new MockResponse()
    //        .setResponseCode(HttpStatus.CONTINUE.value())
    //        .setHeader("Content-Type", "application/json")
    //        .setBody("Hallo Welt!"));
    //
    //try
    //{
    //  String result = clientTemplate.getForObject(uri, String.class);
    //  assertEquals("Hallo Welt!", result);
    //}
    //catch(Exception e)
    //{
    //  LOG.debug("{}", e.toString());
    //  fail("Unexpected error: " + e);
    //}


    server
        .enqueue(new MockResponse()
            .setResponseCode(HttpStatus.OK.value())
            .setHeader("Content-Type", "text/plain")
            .setBody("Hallo Welt!"));

    try
    {
      String result = clientTemplate.getForObject(uri, String.class);
      assertEquals("Hallo Welt!", result);
    }
    catch(Exception e)
    {
      LOG.debug("{}", e.toString());
      fail("Unexpected error: " + e);
    }


    server
        .enqueue(new MockResponse()
            .setResponseCode(HttpStatus.TEMPORARY_REDIRECT.value())
            .setHeader("Content-Type", "text/plain")
            .setBody("Hallo Welt!"));

    try
    {
      String result = clientTemplate.getForObject(uri, String.class);
      assertEquals("Hallo Welt!", result);
    }
    catch(Exception e)
    {
      LOG.debug("{}", e.toString());
      fail("Unexpected error: " + e);
    }


    server
        .enqueue(new MockResponse()
            .setResponseCode(HttpStatus.INTERNAL_SERVER_ERROR.value())
            .setHeader("Content-Type", "text/plain")
            .setBody("Hallo Welt!"));

    try
    {
      clientTemplate.getForObject(uri, String.class);
      fail("The parent handler should have raised an exception!");
    }
    catch(HttpServerErrorException e)
    {
      LOG.debug("Expexted error: {}", e.toString());
    }
    catch(Exception e)
    {
      LOG.debug("{}", e.toString());
      fail("Unexpected error: " + e);
    }
  }

  @Test
  public void testValidError()
  {
    server
        .enqueue(new MockResponse()
            .setResponseCode(HttpStatus.BAD_REQUEST.value())
            .setHeader("Content-Type", "application/json")
            .setBody(
                "{\n" +
                "  \"error\":\n" +
                "  {\n" +
                "    \"message\": \"(#613) Calls to stream have exceeded the rate of 600 calls per 600 seconds.\",\n" +
                "    \"type\": \"OAuthException\",\n" +
                "    \"code\": 613\n" +
                "  }\n" +
                "}"));

    try
    {
      clientTemplate.getForObject(uri, String.class);
      fail("The expected exception was not thrown");
    }
    catch(RateLimitExceededException e)
    {
      LOG.debug("{}", e.toString());
      assertEquals(new Integer(613), e.getCode());
      assertEquals("(#613) Calls to stream have exceeded the rate of 600 calls per 600 seconds.", e.getMessage());
      assertEquals(Type.OAuthException, e.getType());
    }
  }

  @Test
  public void testUnmappedError()
  {
    server
        .enqueue(new MockResponse()
            .setResponseCode(HttpStatus.BAD_REQUEST.value())
            .setHeader("Content-Type", "application/json")
            .setBody(
                "{\n" +
                "  \"error\":\n" +
                "  {\n" +
                "    \"message\": \"This error does not exist.\",\n" +
                "    \"type\": \"NonexistentTypeException\",\n" +
                "    \"code\": 999999999\n" +
                "  }\n" +
                "}"));

    try
    {
      clientTemplate.getForObject(uri, String.class);
      fail("The expected exception was not thrown");
    }
    catch(GraphApiException e)
    {
      LOG.debug("{}", e.toString());
      assertEquals(new Integer(999999999), e.getCode());
      assertEquals("This error does not exist.", e.getMessage());
      try
      {
        Type type = e.getType();
        LOG.error("unknown type: {}", type);
        fail("unmapped type was resolved by enum: " + type);
      }
      catch (IllegalArgumentException ee) {}
    }
  }

  @Test
  public void testInvlalidError()
  {
    server
        .enqueue(new MockResponse()
            .setResponseCode(HttpStatus.BAD_REQUEST.value())
            .setHeader("Content-Type", "application/json")
            .setBody(
                "{\n" +
                "  \"error\":\n" +
                "  {\n" +
                "    \"message\": \"Not a Graph-Api-Exception.\",\n" +
                "    \"type\": \"Whatever\",\n" +
                "    \"code\": \"some string\"\n" +
                "  }\n" +
                "}"));

    try
    {
      clientTemplate.getForObject(uri, String.class);
      fail("The parent handler should have raised an exception!");
    }
    catch(HttpClientErrorException e)
    {
      LOG.debug("Expexted error: {}", e.toString());
    }
    catch(Exception e)
    {
      LOG.debug("{}", e.toString());
      fail("Unexpected error: " + e);
    }


    server
        .enqueue(new MockResponse()
            .setResponseCode(HttpStatus.BAD_REQUEST.value())
            .setHeader("Content-Type", "text/plain")
            .setBody("Hallo Welt!"));

    try
    {
      clientTemplate.getForObject(uri, String.class);
      fail("The parent handler should have raised an exception!");
    }
    catch(HttpClientErrorException e)
    {
      LOG.debug("Expexted error: {}", e.toString());
    }
    catch(Exception e)
    {
      LOG.debug("{}", e.toString());
      fail("Unexpected error: " + e);
    }
  }
}
